==> alwaysai-2.3.4.tgz: package/src/components/project/project-select-component.ts <==
import download = require('download');
import tmp = require('tmp');
import * as logSymbols from 'log-symbols';
import * as chalk from 'chalk';
import { join } from 'path';

import { Choice } from 'prompts';
import { CliTerseError, CliUsageError } from '@alwaysai/alwayscli';
import { copySync, readdirSync, removeSync } from 'fs-extra';
import { PLEASE_REPORT_THIS_ERROR_MESSAGE } from '../../constants';
import { RequiredWithYesMessage, promptForInput, logger, echo } from '../../util';
import { confirmPromptComponent } from '../general';
import { getProjectByUUID } from '../../core/project';
import { CliAuthenticationClient, CliRpcClient } from '../../infrastructure';

const STARTER_APPS = [
  {
    webName: 'realtime_object_detector',
    purpose: 'Object Detection',
    downloadUrl: 'https://github.com/alwaysai/object-detector/archive/main.zip',
  },
  {
    webName: 'detector_tracker',
    purpose: 'Object Detection with Tracking',
    downloadUrl: 'https://github.com/alwaysai/detector-tracker/archive/main.zip',
  },
  {
    webName: 'image_classifier',
    purpose: 'Image Classification',
    downloadUrl: 'https://github.com/alwaysai/image-classifier/archive/main.zip',
  },
  {
    webName: 'semantic_segmentation_voc',
    purpose: 'Semantic Segmentation',
    downloadUrl: 'https://github.com/alwaysai/semantic-segmentation-voc/archive/main.zip',
  },
  {
    webName: 'realtime_pose_estimator',
    purpose: 'Human Pose Estimation',
    downloadUrl: 'https://github.com/alwaysai/pose-estimator/archive/main.zip',
  },
  {
    webName: 'hello_world_face_detector',
    purpose: 'Face Detector',
    downloadUrl: 'https://github.com/alwaysai/face-detector/archive/main.zip',
  },
  {
    webName: 'instance_segmentation',
    purpose: 'Instance Segmentation',
    downloadUrl: 'https://github.com/alwaysai/instance-segmentation/archive/main.zip',
  },
];

export async function projectSelectComponent(props: {
  yes: boolean;
  projectUuid?: string;
}) {
  const { yes, projectUuid } = props;
  if (yes && !projectUuid) {
    throw new CliUsageError(RequiredWithYesMessage('project', undefined));
  }
  const project = projectUuid
    ? await getProjectByUUID(projectUuid)
    : await selectFromListOfProjects();
  if (project.starter_app) {
    let appName = project.starter_app;

    // FIXME: Replace legacy "hello_world" starter app with face detector on outdated projects
    if (appName === 'hello_world') {
      appName = 'hello_world_face_detector';
    }

    await downloadStarterApp({ yes, appName });
  }
  return project;
}

async function selectFromListOfProjects() {
  const authInfo = await CliAuthenticationClient().getInfo();

  const projects = await CliRpcClient().listProjectsUserIsCollaborator({
    user_name: authInfo.username,
  });

  const choices: Choice[] = [
    { title: chalk.green.bold('Create new project'), value: 'new' },
  ];
  projects.forEach((project, index) => {
    choices.push({
      title: project.name,
      value: `${index}`,
    });
  });

  const choice = await promptForInput({
    purpose: 'to select a project',
    questions: [
      {
        type: 'select',
        name: 'project',
        message: 'Select a project or create a new project',
        initial: 0,
        choices,
      },
    ],
  });

  if (choice.project === 'new') {
    return await createProject();
  }
  return projects[choice.project];
}

async function createProject() {
  const { username } = await CliAuthenticationClient().getInfo();
  const team = await CliRpcClient().listTeamsUserIsOwner({ user_name: username });

  // TODO: Need to limit input characters to match dashboard implementation
  const input = await promptForInput({
    purpose: 'to choose a project name',
    questions: [
      {
        type: 'text',
        name: 'name',
        message: 'Enter a project name:',
      },
    ],
  });
  const name = input.name;

  const choice = await promptForInput({
    purpose: 'to select a project type',
    questions: [
      {
        type: 'select',
        name: 'type',
        message: 'How would you like to initialize your project?',
        initial: 0,
        choices: [
          { title: 'From a Starter App', value: 'starterApp' },
          { title: 'As an empty app', value: 'empty' },
        ],
      },
    ],
  });

  let starterApp = '';

  switch (choice.type) {
    case 'starterApp': {
      const choices: Choice[] = [];
      STARTER_APPS.forEach((app) => {
        choices.push({
          title: app.purpose,
          value: app.webName,
        });
      });
      const choice = await promptForInput({
        purpose: 'to select a starter app',
        questions: [
          {
            type: 'select',
            name: 'app',
            message: 'Select a Starter App',
            initial: 0,
            choices,
          },
        ],
      });
      starterApp = choice.app;
      break;
    }
    case 'empty':
    default:
    // Do nothing
  }

  const project = await CliRpcClient().createProject({
    owner: username,
    team_id: team[0].id,
    name,
    description: '',
    git_url: '',
    starter_app: starterApp,
  });

  return project;
}

async function checkDirIsEmpty(dir) {
  const fileNames = readdirSync(dir);
  return fileNames.length === 0;
}

async function downloadStarterApp(props: { yes: boolean; appName: string }) {
  const { yes, appName } = props;
  let url: string | undefined = undefined;
  if ((await checkDirIsEmpty(process.cwd())) === false) {
    const proceed =
      yes ||
      (await confirmPromptComponent({
        message:
          'Would you like to download the Starter App? Files in this directory may be overwritten!',
      }));
    if (!proceed) {
      echo('Skipping download of starter app!');
      return;
    }
  }
  for (const app of STARTER_APPS) {
    if (app.webName === appName) {
      url = app.downloadUrl;
      break;
    }
  }
  if (url === undefined) {
    throw new CliTerseError(`Starter App "${appName}" not found!`);
  }
  const tmpDir = tmp.dirSync();
  try {
    await download(url, tmpDir.name, { extract: true });
  } catch (err) {
    throw new CliTerseError(
      `Failed to download "${url}", check your internet connection and retry`,
    );
  }

  const fileNames = readdirSync(tmpDir.name);
  if (fileNames.length !== 1 || !fileNames[0]) {
    logger.error(`Expected application to contain a single directory: ${fileNames}`);
    throw new CliTerseError(
      `Expected application to contain a single directory. ${PLEASE_REPORT_THIS_ERROR_MESSAGE}`,
    );
  }

  const destDir = process.cwd();
  const srcDir = join(tmpDir.name, fileNames[0]);
  const starterAppFiles = readdirSync(srcDir);
  starterAppFiles.forEach((file) => {
    const src = join(srcDir, file);
    const dest = join(destDir, file);
    copySync(src, dest);
  });
  removeSync(srcDir);
  tmpDir.removeCallback();
  echo(`${logSymbols.success} Download Starter App`);
}

==> alwaysai-2.5.1.tgz: package/src/components/project/project-select-component.ts <==
import download = require('download');
import tmp = require('tmp');
import * as logSymbols from 'log-symbols';
import * as chalk from 'chalk';
import { join } from 'path';

import { Choice } from 'prompts';
import { CliTerseError, CliUsageError } from '@alwaysai/alwayscli';
import { copySync, readdirSync, removeSync } from 'fs-extra';
import { PLEASE_REPORT_THIS_ERROR_MESSAGE } from '../../constants';
import {
  RequiredWithYesMessage,
  promptForInput,
  logger,
  echo
} from '../../util';
import { confirmPromptComponent } from '../general';
import { getProjectByUUID } from '../../core/project';
import { CliAuthenticationClient, CliRpcClient } from '../../infrastructure';

const STARTER_APPS = [
  {
    webName: 'realtime_object_detector',
    purpose: 'Object Detection',
    downloadUrl: 'https://github.com/alwaysai/object-detector/archive/main.zip'
  },
  {
    webName: 'detector_tracker',
    purpose: 'Object Detection with Tracking',
    downloadUrl: 'https://github.com/alwaysai/detector-tracker/archive/main.zip'
  },
  {
    webName: 'image_classifier',
    purpose: 'Image Classification',
    downloadUrl: 'https://github.com/alwaysai/image-classifier/archive/main.zip'
  },
  {
    webName: 'semantic_segmentation_voc',
    purpose: 'Semantic Segmentation',
    downloadUrl:
      'https://github.com/alwaysai/semantic-segmentation-voc/archive/main.zip'
  },
  {
    webName: 'realtime_pose_estimator',
    purpose: 'Human Pose Estimation',
    downloadUrl: 'https://github.com/alwaysai/pose-estimator/archive/main.zip'
  },
  {
    webName: 'hello_world_face_detector',
    purpose: 'Face Detector',
    downloadUrl: 'https://github.com/alwaysai/face-detector/archive/main.zip'
  },
  {
    webName: 'instance_segmentation',
    purpose: 'Instance Segmentation',
    downloadUrl:
      'https://github.com/alwaysai/instance-segmentation/archive/main.zip'
  }
];

export async function projectSelectComponent(props: {
  yes: boolean;
  projectUuid?: string;
}) {
  const { yes, projectUuid } = props;
  if (yes && !projectUuid) {
    throw new CliUsageError(RequiredWithYesMessage('project', undefined));
  }
  const project = projectUuid
    ? await getProjectByUUID(projectUuid)
    : await selectFromListOfProjects();
  if (project.starter_app) {
    let appName = project.starter_app;

    // FIXME: Replace legacy "hello_world" starter app with face detector on outdated projects
    if (appName === 'hello_world') {
      appName = 'hello_world_face_detector';
    }

    await downloadStarterApp({ yes, appName });
  }
  return project;
}

async function selectFromListOfProjects() {
  const authInfo = await CliAuthenticationClient().getInfo();

  const projects = await CliRpcClient().listProjectsUserIsCollaborator({
    user_name: authInfo.username
  });

  const choices: Choice[] = [
    { title: chalk.green.bold('Create new project'), value: 'new' }
  ];
  projects.forEach((project, index) => {
    choices.push({
      title: project.name,
      value: `${index}`
    });
  });

  const choice = await promptForInput({
    purpose: 'to select a project',
    questions: [
      {
        type: 'select',
        name: 'project',
        message: 'Select a project or create a new project',
        initial: 0,
        choices
      }
    ]
  });

  if (choice.project === 'new') {
    return await createProject();
  }
  return projects[choice.project];
}

async function createProject() {
  const { username } = await CliAuthenticationClient().getInfo();
  const team = await CliRpcClient().listTeamsUserIsOwner({
    user_name: username
  });

  // TODO: Need to limit input characters to match dashboard implementation
  const input = await promptForInput({
    purpose: 'to choose a project name',
    questions: [
      {
        type: 'text',
        name: 'name',
        message: 'Enter a project name:'
      }
    ]
  });
  const name = input.name;

  const choice = await promptForInput({
    purpose: 'to select a project type',
    questions: [
      {
        type: 'select',
        name: 'type',
        message: 'How would you like to initialize your project?',
        initial: 0,
        choices: [
          { title: 'From a Starter App', value: 'starterApp' },
          { title: 'As an empty app', value: 'empty' }
        ]
      }
    ]
  });

  let starterApp = '';

  switch (choice.type) {
    case 'starterApp': {
      const choices: Choice[] = [];
      STARTER_APPS.forEach((app) => {
        choices.push({
          title: app.purpose,
          value: app.webName
        });
      });
      const choice = await promptForInput({
        purpose: 'to select a starter app',
        questions: [
          {
            type: 'select',
            name: 'app',
            message: 'Select a Starter App',
            initial: 0,
            choices
          }
        ]
      });
      starterApp = choice.app;
      break;
    }
    case 'empty':
    default:
    // Do nothing
  }

  const project = await CliRpcClient().createProject({
    owner: username,
    team_id: team[0].id,
    name,
    description: '',
    git_url: '',
    starter_app: starterApp
  });

  return project;
}

async function checkDirIsEmpty(dir) {
  const fileNames = readdirSync(dir);
  return fileNames.length === 0;
}

async function downloadStarterApp(props: { yes: boolean; appName: string }) {
  const { yes, appName } = props;
  let url: string | undefined = undefined;
  if ((await checkDirIsEmpty(process.cwd())) === false) {
    const proceed =
      yes ||
      (await confirmPromptComponent({
        message:
          'Would you like to download the Starter App? Files in this directory may be overwritten!'
      }));
    if (!proceed) {
      echo('Skipping download of starter app!');
      return;
    }
  }
  for (const app of STARTER_APPS) {
    if (app.webName === appName) {
      url = app.downloadUrl;
      break;
    }
  }
  if (url === undefined) {
    throw new CliTerseError(`Starter App "${appName}" not found!`);
  }
  const tmpDir = tmp.dirSync();
  try {
    await download(url, tmpDir.name, { extract: true });
  } catch (err) {
    throw new CliTerseError(
      `Failed to download "${url}", check your internet connection and retry`
    );
  }

  const fileNames = readdirSync(tmpDir.name);
  if (fileNames.length !== 1 || !fileNames[0]) {
    logger.error(
      `Expected application to contain a single directory: ${fileNames}`
    );
    throw new CliTerseError(
      `Expected application to contain a single directory. ${PLEASE_REPORT_THIS_ERROR_MESSAGE}`
    );
  }

  const destDir = process.cwd();
  const srcDir = join(tmpDir.name, fileNames[0]);
  const starterAppFiles = readdirSync(srcDir);
  starterAppFiles.forEach((file) => {
    const src = join(srcDir, file);
    const dest = join(destDir, file);
    copySync(src, dest);
  });
  removeSync(srcDir);
  tmpDir.removeCallback();
  echo(`${logSymbols.success} Download Starter App`);
}
